'use strict';

const puppeteer = require('puppeteer');

const testLogger = require('./lib/logger');
const testReporter = require('./lib/reporter');
const { ...testFunctions } = require('./lib/tests');
const { createLinkChecker } = require('./lib/linkUtils');

// eslint-disable-next-line max-lines-per-function
const executeAllTests = async (config) => {
    const logger = testLogger(config);
    const linkChecker = createLinkChecker();
    const browser = await puppeteer.launch(config.puppeteerLaunchOptions);

    for (const testUrl of config.urls) {
        logger.startUrlTests(testUrl.rawUrl);

        const consoleLog = [];
        const page = await browser.newPage();
        // Object property names defined by puppeteer API
        // eslint-disable-next-line no-underscore-dangle
        page.on('console', msg => consoleLog.push({ _type: msg._type, _text: msg._text, _stackTraceLocations: msg._stackTraceLocations }));
        await page.goto(testUrl.url, { waitUntil: 'load' });

        const testContext = { page, consoleLog,
            urlSettings: { ...testUrl.settings, htmlHintConfig: config.htmlHintConfig }, logger, linkChecker };
        for (const test of Object.keys(testFunctions)) {
            await testFunctions[test](testContext);
        }

        await page.close();
    }
    await browser.close();
    const testResults = logger.getTestResults();
    testReporter.saveReports(testResults, config.reporters);

    if (testResults.summary.failed > 0) {
        // For test harness want process to exit with error code
        // eslint-disable-next-line no-process-exit, no-magic-numbers
        process.exit(2);
    }
};

module.exports.executeAllTests = executeAllTests;
